/*
**  SMTP.h
**
**  Copyright (c) 2001-2004
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**  
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
**  Lesser General Public License for more details.
**  
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef _Pantomime_H_SMTP
#define _Pantomime_H_SMTP

#include <Pantomime/Service.h>
#include <Pantomime/Transport.h>

#import <Foundation/NSArray.h>
#import <Foundation/NSData.h>
#import <Foundation/NSObject.h>
#import <Foundation/NSString.h>

@class Message;

/*!
  @typedef SMTPCommand
  @abstract Supported SMTP commands.
  @discussion This enum lists the supported SMTP commands available
              in Pantomime's SMTP client code.
  @constant SMTP_AUTH_CRAM_MD5 CRAM-MD5 authentication.
  @constant SMTP_AUTH_LOGIN LOGIN authentication.
  @constant SMTP_AUTH_LOGIN_CHALLENGE Challenge during the LOGIN authentication.
  @constant SMTP_AUTH_PLAIN PLAIN authentication.
  @constant SMTP_DATA The DATA SMTP command - see 4.1.1.4 DATA (DATA) of RFC 2821.
  @constant SMTP_EHLO The EHLO SMTP command - see 4.1.1.1  Extended HELLO (EHLO) or HELLO (HELO) of RFC 2821.
  @constant SMTP_HELO The HELO SMTP command - see 4.1.1.1  Extended HELLO (EHLO) or HELLO (HELO) of RFC 2821.
  @constant SMTP_MAIL The MAIL SMTP command - see 4.1.1.2 MAIL (MAIL) of RFC 2821.
  @constant SMTP_NOOP The NOOP SMTP command - see 4.1.1.9 NOOP (NOOP) of RFC 2821.
  @constant SMTP_QUIT The QUIT SMTP command - see 4.1.1.10 QUIT (QUIT) of RFC 2821.
  @constant SMTP_RCPT The RCPT SMTP command - see 4.1.1.3 RECIPIENT (RCPT) of RFC 2821.
  @constant SMTP_RSET The RSET SMTP command - see 4.1.1.5 RESET (RSET) of RFC 2821.
  @constant SMTP_AUTHORIZATION Special command to know we are in the autorization state.
  @constant SMTP_EMPTY_QUEUE Special command to empty the command queue.
*/
typedef enum {
  SMTP_AUTH_CRAM_MD5,
  SMTP_AUTH_LOGIN,
  SMTP_AUTH_LOGIN_CHALLENGE,
  SMTP_AUTH_PLAIN,
  SMTP_DATA,
  SMTP_EHLO,
  SMTP_HELO,
  SMTP_MAIL,
  SMTP_NOOP,
  SMTP_QUIT,
  SMTP_RCPT,
  SMTP_RSET,
  SMTP_AUTHORIZATION,
  SMTP_EMPTY_QUEUE,
} SMTPCommand;

/*!
  @class SMTP
  @abstract Pantomime SMTP client code.
  @discussion This class, which extends the Service class and implements
              the Transport protocol, is Pantomime's SMTP client code.
*/
@interface SMTP : Service <Transport>
{
  @private
    NSMutableArray *_recipients;
    Message *_message;
    NSData *_data;
    
    SMTPCommand _lastCommand;

    unsigned int _max_size;
    BOOL _redirected;
}

/*!
  @method lastResponse
  @discussion This method is used to obtain the last response
              received from the SMTP server. If the server
	      sent a multi-line response, only the last line
	      will be returned.
  @result The last response in its complete form, nil if no
          response was read.
*/
- (NSData *) lastResponse;

/*!
  @method lastResponseCode
  @discussion This method is used to obtain the last response code
              received from the SMTP server. If the server
	      sent a multi-line response, only the code of the
	      last line will be returned.
  @result The last response code in its complete form, 0 if
          no response was read.         
*/
- (int) lastResponseCode;

/*!
  @method reset
  @discussion This method is used to send the RSET SMTP command.
              See 4.1.1.5 RESET (RSET) of RFC 2821 for a full description
	      of this command.
*/
- (void) reset;

/*!
  @method sendCommand: arguments: ...
  @discussion This method is used to send commands to the SMTP server.
              Normally, you should not call this method directly.
  @param theCommand The SMTP command to send.
  @param theFormat The format defining the variable arguments list.
*/
- (void) sendCommand: (SMTPCommand) theCommand  arguments: (NSString *) theFormat, ...;

@end

#endif // _Pantomime_H_SMTP
