/*
**  Service.h
**
**  Copyright (c) 2001-2004
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**  
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
**  Lesser General Public License for more details.
**  
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef _Pantomime_H_Service
#define _Pantomime_H_Service

#include <Pantomime/Connection.h>

#import <Foundation/NSArray.h>
#import <Foundation/NSObject.h>
#import <Foundation/NSRunLoop.h>
#import <Foundation/NSTimer.h>

#ifdef MACOSX
#import <Foundation/NSMapTable.h>
#include <CoreFoundation/CoreFoundation.h>
#endif

/*!
  @function split_lines
  @discussion This function is used to split lines from <i>theMutableData</i> and
              return the first one found immediately (by removing it first
              from <i>theMutableData</i>). IMAP, POP3 and SMTP servers reponses are 
              always ended by CRLF.
  @param theMutableData The data from which to split lines from.
  @result A line as a NSData instance, nil if no line was splitted.
*/
static inline NSData *split_lines(NSMutableData *theMutableData)
{
  char *bytes, *end;
  int i, count;

  end = bytes = (char *)[theMutableData mutableBytes];
  count = [theMutableData length];

  for (i = 0; i < count; i++)
    {
      if (*end == '\n' && *(end-1) == '\r')
	{
	  NSData *aData;
	  
	  aData = [NSData dataWithBytes: bytes  length: (i-1)];
	  memmove(bytes,end+1,count-i-1);
	  [theMutableData setLength: count-i-1];
	  
	  return aData;
	}

      end++;
    }

  return nil;
}


@class NSString;

#ifdef MACOSX
typedef enum {ET_RDESC, ET_WDESC, ET_EDESC} RunLoopEventType;

/*!
  @class Service
  @discussion This abstract class defines the basic behavior and implementation
              of all Pantomime internet services such as SMTP, POP3 and IMAP.
	      You should never instantiate this class directly. You rather
	      need to instantiate the SMTP, POP3Store or IMAPStore classes,
	      which fully implement the abstract methods found in this class.
*/
@interface Service : NSObject
#else
@interface Service : NSObject <RunLoopEvents>
#endif
{
  @protected
    NSMutableArray *_supportedMechanisms;
    NSMutableArray *_responsesFromServer;
    NSMutableArray *_queue;
    NSMutableArray *_runLoopModes ;
    NSMutableData *_wbuf;
    NSMutableData *_rbuf;
    NSString *_username;
    NSString *_password;
    NSString *_name;

#ifdef MACOSX
    CFRunLoopSourceRef _runLoopSource;
    CFSocketContext *_context;
    CFSocketRef _socket;
#endif

    unsigned int _port;
    BOOL _reconnecting;
    BOOL _connected;
    id _delegate;
    
    id<Connection> _connection;
    NSTimer * _timer;
    int _counter;
}

/*!
  @method initWithName: port:
  @discussion This is the designated initializer for the Service class.
              Once called, it'll open a connection to the server specified
	      by <i>theName</i> using the specified port (<i>thePort</i>).
  @param theName The server DNS name.
  @param thePort The server port to which we will connect.
  @result An instance of a Service class, nil on error.
*/
- (id) initWithName: (NSString *) theName
               port: (unsigned int) thePort;

/*!
  @method setDelegate:
  @discussion This method is used to set the Service's delegate.
              The delegate will not be retained. The Service class
	      (and its subclasses) will invoke methods on the delegate
	      based on actions performed.
  @param theDelegate The delegate, which implements various callback methods.
*/
- (void) setDelegate: (id) theDelegate;

/*!
  @method delegate
  @discussion This method is used to get the delegate of the Service's instance.
  @result The delegate, nil if none was previously set.
*/
- (id) delegate;

/*!
  @method name
  @discussion This method is used to obtain the server name.
  @result The server name.
*/
- (NSString *) name;

/*!
  @method setName:
  @discussion This method is used to set the server name to which
              we will eventually connect to.
  @param theName The name of the server.
*/
- (void) setName: (NSString *) theName;

/*!
  @method port
  @discussion This method is used to obtain the server port.
  @result The server port.
*/
- (unsigned int) port;

/*!
  @method setPort:
  @discussion This method is used to set the server port to which
              we will eventually connect to.
  @param theName The port of the server.
*/
- (void) setPort: (unsigned int) thePort;

/*!
  @method connection
  @discussion This method is used to retrieve the associated connection
              object for the service (usually a TCPConnection instance).
  @result The associated connectio object.
*/
- (id<Connection>) connection;

/*!
  @method username
  @discussion This method is used to get the username (if any) that will be
              used to authenticate to the service.
  @result The username.
*/
- (NSString *) username;

/*!
  @method setUsername:
  @discussion This method is used to set the username that will be used
              to authenticate to the service.
  @param theUsername The username for authentication.
*/
- (void) setUsername: (NSString *) theUsername;

/*!
  @method supportedMechanisms
  @discussion This method is used to return the supported SASL
              authentication mecanisms by the receiver.
  @result An array of NSString instances which indicates
          what SASL mechanisms are supported.
*/
- (NSArray *) supportedMechanisms;

/*!
  @method isConnected
  @discussion This method is used to verify if the receiver
              is connected to the server.
  @result YES if connected, NO otherwise.
*/
- (BOOL) isConnected;

/*!
  @method authenticate: password: mechanism:
  @discussion This method is used to authentifcate the receiver
              to the server. This method posts a PantomimeAuthenticationCompleted
	      (or calls the -authenticationCompleted: method on the delegate, if any)
	      if the authentication is sucessful. If not, it posts the
	      PantomimeAuthenticationFailed notification (or calls the
	      -authenticationFailed: method on the delegate, if any). This method
	      is fully asynchronous.
  @param theUsername The username to use, overwriting -username: if any.
  @param thePassword The password to use.
  @param theMechanism The authentication mechanism to use.
*/
- (void) authenticate: (NSString *) theUsername
             password: (NSString *) thePassword
            mechanism: (NSString *) theMechanism;

/*!
  @method cancelRequest
  @discussion This method will cancel any pending requests or communications
              with the server and close the connection. It'll post a
	      PantomimeRequestCancelled once it has fully cancelled everything.
	      This method is fully asynchronous.
*/
- (void) cancelRequest;

/*!
  @method close
  @discussion This method is used to close the connection to the server.
              If the receiver is not in a connected state, it does nothing.
	      If it is, it posts a PantomimeConnectionTerminated notification
	      once it has completed and invokes -connectionTerminated: on the
	      delegate, if any.
*/
- (void) close;

/*!
  @method connect
  @discussion This method is used to connect the receiver to the server.
              It will block until the connection was succefully established
	      (or until it fails).
  @result 0 on success, -1 on error.
*/
- (int) connect;

/*!
  @method connectInBackgroundAndNotify
  @discussion This method is used  connect the receiver to the server.
              The call to this method is non-blocking. This method will
	      post a PantomimeConnectionEstablished notification once
	      the connection has been establish (and call -connectionEstablished:
	      on the delegate, if any). Otherwise, it will post a PantomimeConnectionTimedOut
	      notification (and call -connectionTimedOut: on the delegate, if any).
*/
- (void) connectInBackgroundAndNotify;

/*!
  @method noop
  @discussion This method is used to generate some traffic on a server
              so the connection doesn't idle and gets terminated by
	      the server. Subclasses of Service need to implement this method.
*/
- (void) noop;

- (void) receivedEvent: (void *) theData
                  type: (RunLoopEventType) theType
                 extra: (void *) theExtra
               forMode: (NSString *) theMode;

/*!
  @method reconnect
  @discussion Pending.
  @result Pending.
*/       
- (int) reconnect;

/*!
  @method updateRead
  @discussion This method is invoked automatically when bytes are available
              to be read. You should never have to invoke this method directly.
*/
- (void) updateRead;

/*!
  @method updateWrite
  @discussion This method is invoked automatically when bytes are available
              to be written. You should never have to invoke this method directly.
*/
- (void) updateWrite;

/*!
  @method writeData:
  @discussion This method is used to buffer bytes to be written on the socket.
              You should never have to invoke this method directly.
  @param The bytes to buffer, as a NSData instance.
*/
- (void) writeData: (NSData *) theData;

/*!
  @method addRunLoopMode:
  @discussion This method is used to add an additional mode that the run-loop
              will use to listen for network events for reading and writing.
  @param The additional mode. NSDefaultRunLoopMode is always present so there
         is no need to add it.
*/
- (void) addRunLoopMode: (NSString *) theMode;

@end

#endif // _Pantomime_H_Service
