/*
**  Transport.h
**
**  Copyright (c) 2001-2004
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**  
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
**  Lesser General Public License for more details.
**  
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef _Pantomime_H_Transport
#define _Pantomime_H_Transport

#import <Foundation/NSArray.h>
#import <Foundation/NSData.h>

@class Message;

/*!
  @protocol Transport
  @discussion This protocol defines a generic interface for sending messages.
              The SMTP and Sendmail classes fully implement this protocol.
*/
@protocol Transport

/*!
  @method sendMessage
  @discussion This method sends the message previously set with -setMessage:
              or -setMessageData:. It posts a PantomimeTransportMessageNotSent
	      notification (and calls -messageNotSent: on the delegate, if any)
	      if the message could not be sent. It ports a PantomimeTransportMessageSent
	      notification (and calls -messageSent: on the delegate, if any)
	      if the message has been successfully sent. This method is
	      fully asynchronous.
*/
- (void) sendMessage;

/*!
  @method setMessage:
  @discussion This method is used to set the message (instance of the Message class)
              that the transport class will eventually send (by calling -sendMessage).
  @param theMessage The message to send.
*/
- (void) setMessage: (Message *) theMessage;

/*!
  @method message
  @discussion This method is used to obtain the Message instance
              previously set by calling -setMessage:
  @result The Message instance.
*/
- (Message *) message;

/*!
  @method setMessageData:
  @discussion This method is used to set the message (as raw source) that the transport
              class will eventually send (by calling -sendMessage).
  @param theMessage The message to send.
*/
- (void) setMessageData: (NSData *) theData;

/*!
  @method messageData
  @discussion This method is used to obtain the message (its raw representation)
              previously set by calling -setMessageData:
  @result The message's raw representation.
*/
- (NSData *) messageData;

/*!
  @method setRecipients:
  @discussion This method is used to specify the recipients of the
              message.
  @param theRecipients An NSArray instance which contains InternetAddress
                       instances for all recipients.
*/
- (void) setRecipients: (NSArray *) theRecipients;

/*!
  @method recipients
  @discussion This method is used to obtain the recipients set when
              calling -setRecipients:.
  @result The array of recipients.
*/
- (NSArray *) recipients;

@end

#endif // _Pantomime_H_Transport
